@@ -1,7 +1,7 @@
 /*
  * irc-list.c - functions for IRC list buffer
  *
- * Copyright (C) 2023 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2023-2024 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -22,6 +22,7 @@
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
+#include <sys/stat.h>
 #include <time.h>
 
 #include "../weechat-plugin.h"
@@ -101,12 +102,9 @@ irc_list_free_cb (void *data, struct t_arraylist *arraylist, void *pointer)
     ptr_channel = (struct t_irc_list_channel *)pointer;
     if (ptr_channel)
     {
-        if (ptr_channel->name)
-            free (ptr_channel->name);
-        if (ptr_channel->name2)
-            free (ptr_channel->name2);
-        if (ptr_channel->topic)
-            free (ptr_channel->topic);
+        free (ptr_channel->name);
+        free (ptr_channel->name2);
+        free (ptr_channel->topic);
         free (ptr_channel);
     }
 }
@@ -255,8 +253,7 @@ irc_list_channel_match_filter (struct t_irc_server *server,
             irc_list_filter_hashtable_extra_vars,
             irc_list_filter_hashtable_options);
         match = (result && (strcmp (result, "1") == 0)) ? 1 : 0;
-        if (result)
-            free (result);
+        free (result);
         return match;
     }
 
@@ -451,10 +448,8 @@ irc_list_parse_messages (struct t_irc_server *server, const char *output)
                 weechat_arraylist_add (server->list->channels, channel);
             }
         }
-        if (command)
-            free (command);
-        if (params)
-            weechat_string_free_split (params);
+        free (command);
+        weechat_string_free_split (params);
     }
 
     weechat_string_free_split (irc_msgs);
@@ -556,22 +551,33 @@ irc_list_display_line (struct t_irc_server *server, int line)
 void
 irc_list_buffer_refresh (struct t_irc_server *server, int clear)
 {
-    int num_channels, i;
+    int num_channels, num_filter_channels, i;
 
     if (!server || !server->list->buffer)
         return;
 
-    num_channels = weechat_arraylist_size (server->list->filter_channels);
+    num_channels = weechat_arraylist_size (server->list->channels);
+    num_filter_channels = weechat_arraylist_size (server->list->filter_channels);
 
-    if (clear)
+    if (clear || (num_channels == 0))
     {
         weechat_buffer_clear (server->list->buffer);
         server->list->selected_line = 0;
     }
 
-    for (i = 0; i < num_channels; i++)
+    if (num_channels == 0)
     {
-        irc_list_display_line (server, i);
+        weechat_printf_y (
+            server->list->buffer, 1,
+            "%s",
+            _("Empty list of channels, try \"$\" to refresh list"));
+    }
+    else
+    {
+        for (i = 0; i < num_filter_channels; i++)
+        {
+            irc_list_display_line (server, i);
+        }
     }
 
     irc_list_buffer_set_title (server);
@@ -961,8 +967,7 @@ irc_list_create_buffer (struct t_irc_server *server)
         &irc_input_data_cb, NULL, NULL,
         &irc_buffer_close_cb, NULL, NULL);
 
-    if (buffer_props)
-        weechat_hashtable_free (buffer_props);
+    weechat_hashtable_free (buffer_props);
 
     irc_list_buffer_set_localvar_filter (buffer, server);
 
@@ -1050,6 +1055,83 @@ irc_list_hsignal_redirect_list_cb (const void *pointer,
     return WEECHAT_RC_OK;
 }
 
+/*
+ * Exports channels currently displayed in /list buffer.
+ *
+ * Returns:
+ *   1: export OK
+ *   0: error
+ */
+
+int
+irc_list_export (struct t_irc_server *server, const char *filename)
+{
+    int num_filter_channels, i;
+    char *filename2, *line;
+    FILE *file;
+    struct t_irc_list_channel *ptr_channel;
+    struct t_hashtable *hashtable_pointers, *hashtable_extra_vars;
+
+    if (!server || !server->list->buffer)
+        return 0;
+
+    filename2 = weechat_string_expand_home (filename);
+    if (!filename2)
+        return 0;
+
+    file = fopen (filename2, "w");
+    if (!file)
+    {
+        free (filename2);
+        return 0;
+    }
+
+    fchmod (fileno (file), 0600);
+
+    hashtable_pointers = weechat_hashtable_new (
+        8,
+        WEECHAT_HASHTABLE_STRING,
+        WEECHAT_HASHTABLE_POINTER,
+        NULL, NULL);
+    hashtable_extra_vars = weechat_hashtable_new (
+        128,
+        WEECHAT_HASHTABLE_STRING,
+        WEECHAT_HASHTABLE_STRING,
+        NULL, NULL);
+
+    weechat_hashtable_set (hashtable_pointers, "irc_server", server);
+
+    num_filter_channels = weechat_arraylist_size (server->list->filter_channels);
+    for (i = 0; i < num_filter_channels; i++)
+    {
+        ptr_channel = (struct t_irc_list_channel *)weechat_arraylist_get (
+            server->list->filter_channels, i);
+        if (!ptr_channel)
+            continue;
+
+        weechat_hashtable_set (hashtable_pointers, "irc_list_channel", ptr_channel);
+
+        irc_list_add_channel_in_hashtable (hashtable_extra_vars, ptr_channel);
+
+        line = weechat_string_eval_expression (
+            weechat_config_string (irc_config_look_list_buffer_format_export),
+            hashtable_pointers,
+            hashtable_extra_vars,
+            NULL);
+        if (line && line[0])
+            fprintf (file, "%s\n", line);
+        free (line);
+    }
+
+    fclose (file);
+
+    weechat_hashtable_free (hashtable_pointers);
+    weechat_hashtable_free (hashtable_extra_vars);
+    free (filename2);
+
+    return 1;
+}
+
 /*
  * Resets lists used by list buffer.
  */
@@ -1223,7 +1305,6 @@ irc_list_mouse_hsignal_cb (const void *pointer, void *data, const char *signal,
 {
     const char *ptr_key, *ptr_chat_line_y, *ptr_buffer_pointer;
     struct t_gui_buffer *ptr_buffer;
-    unsigned long value;
     char str_command[1024];
     int rc;
 
@@ -1239,10 +1320,9 @@ irc_list_mouse_hsignal_cb (const void *pointer, void *data, const char *signal,
     if (!ptr_key || !ptr_buffer_pointer || !ptr_chat_line_y)
         return WEECHAT_RC_OK;
 
-    rc = sscanf (ptr_buffer_pointer, "%lx", &value);
+    rc = sscanf (ptr_buffer_pointer, "%p", &ptr_buffer);
     if ((rc == EOF) || (rc == 0))
         return WEECHAT_RC_OK;
-    ptr_buffer = (struct t_gui_buffer *)value;
     if (!ptr_buffer)
         return WEECHAT_RC_OK;
 
@@ -1340,4 +1420,5 @@ irc_list_end ()
         weechat_hashtable_free (irc_list_filter_hashtable_options);
         irc_list_filter_hashtable_options = NULL;
     }
+    irc_list_hdata_list_channel = NULL;
 }
