@@ -55,42 +55,26 @@ relay_http_request_reinit (struct t_relay_http_request *request)
 {
     request->status = RELAY_HTTP_METHOD;
     weechat_string_dyn_copy (request->raw, NULL);
-    if (request->method)
-    {
-        free (request->method);
-        request->method = NULL;
-    }
-    if (request->path)
-    {
-        free (request->path);
-        request->path = NULL;
-    }
-    if (request->path_items)
-    {
-        weechat_string_free_split (request->path_items);
-        request->path_items = NULL;
-    }
+    free (request->method);
+    request->method = NULL;
+    free (request->path);
+    request->path = NULL;
+    weechat_string_free_split (request->path_items);
+    request->path_items = NULL;
     request->num_path_items = 0;
     weechat_hashtable_remove_all (request->params);
-    if (request->http_version)
-    {
-        free (request->http_version);
-        request->http_version = NULL;
-    }
+    free (request->http_version);
+    request->http_version = NULL;
     weechat_hashtable_remove_all (request->headers);
     weechat_hashtable_remove_all (request->accept_encoding);
-    if (request->ws_deflate)
-    {
-        relay_websocket_deflate_free (request->ws_deflate);
-        request->ws_deflate = relay_websocket_deflate_alloc ();
-    }
+    relay_websocket_deflate_free (request->ws_deflate);
+    request->ws_deflate = relay_websocket_deflate_alloc ();
     request->content_length = 0;
     request->body_size = 0;
-    if (request->body)
-    {
-        free (request->body);
-        request->body = NULL;
-    }
+    free (request->body);
+    request->body = NULL;
+    free (request->id);
+    request->id = NULL;
 }
 
 /*
@@ -132,6 +116,7 @@ relay_http_request_alloc ()
     new_request->content_length = 0;
     new_request->body_size = 0;
     new_request->body = NULL;
+    new_request->id = NULL;
 
     return new_request;
 }
@@ -594,72 +579,82 @@ relay_http_get_auth_status (struct t_relay_client *client)
         goto end;
     }
 
-    auth = weechat_hashtable_get (client->http_req->headers, "authorization");
-    if (!auth || (weechat_strncasecmp (auth, "basic ", 6) != 0))
-    {
-        rc = -1;
-        goto end;
-    }
-
-    pos = auth + 6;
-    while (pos[0] == ' ')
-    {
-        pos++;
-    }
-
-    length = strlen (pos);
-    user_pass = malloc (length + 1);
-    if (!user_pass)
-    {
-        rc = -8;
-        goto end;
-    }
-    length = weechat_string_base_decode ("64", pos, user_pass);
-    if (length < 0)
+    if (!relay_password[0]
+        && !weechat_config_boolean (relay_config_network_allow_empty_password))
     {
         rc = -2;
         goto end;
     }
-    if (strncmp (user_pass, "plain:", 6) == 0)
+
+    if (relay_password[0])
     {
-        switch (relay_auth_check_password_plain (client, user_pass + 6, relay_password))
+        auth = weechat_hashtable_get (client->http_req->headers, "authorization");
+        if (!auth || (weechat_strncasecmp (auth, "basic ", 6) != 0))
         {
-            case 0: /* password OK */
-                break;
-            case -1: /* "plain" is not allowed */
-                rc = -5;
-                goto end;
-            case -2: /* invalid password */
-            default:
-                rc = -2;
-                goto end;
+            rc = -1;
+            goto end;
         }
-    }
-    else if (strncmp (user_pass, "hash:", 5) == 0)
-    {
-        switch (relay_auth_password_hash (client, user_pass + 5, relay_password))
+
+        pos = auth + 6;
+        while (pos[0] == ' ')
         {
-            case 0: /* password OK */
-                break;
-            case -1: /* invalid hash algorithm */
-                rc = -5;
-                goto end;
-            case -2: /* invalid timestamp */
-                rc = -6;
-                goto end;
-            case -3: /* invalid iterations */
-                rc = -7;
-                goto end;
-            case -4: /* invalid password */
-            default:
-                rc = -2;
-                goto end;
+            pos++;
+        }
+
+        length = strlen (pos);
+        user_pass = malloc (length + 1);
+        if (!user_pass)
+        {
+            rc = -8;
+            goto end;
+        }
+        length = weechat_string_base_decode ("64", pos, user_pass);
+        if (length < 0)
+        {
+            rc = -2;
+            goto end;
+        }
+        if (strncmp (user_pass, "plain:", 6) == 0)
+        {
+            switch (relay_auth_check_password_plain (client, user_pass + 6, relay_password))
+            {
+                case 0: /* password OK */
+                    break;
+                case -1: /* "plain" is not allowed */
+                    rc = -5;
+                    goto end;
+                case -2: /* invalid password */
+                default:
+                    rc = -2;
+                    goto end;
+            }
+        }
+        else if (strncmp (user_pass, "hash:", 5) == 0)
+        {
+            switch (relay_auth_password_hash (client, user_pass + 5, relay_password))
+            {
+                case 0: /* password OK */
+                    break;
+                case -1: /* invalid hash algorithm */
+                    rc = -5;
+                    goto end;
+                case -2: /* invalid timestamp */
+                    rc = -6;
+                    goto end;
+                case -3: /* invalid iterations */
+                    rc = -7;
+                    goto end;
+                case -4: /* invalid password */
+                default:
+                    rc = -2;
+                    goto end;
+            }
+        }
+        else
+        {
+            rc = -2;
+            goto end;
         }
-    }
-    else
-    {
-        rc = -2;
-        goto end;
     }
 
     totp_secret = weechat_string_eval_expression (
@@ -1338,6 +1333,7 @@ relay_http_request_free (struct t_relay_http_request *request)
     weechat_hashtable_free (request->accept_encoding);
     relay_websocket_deflate_free (request->ws_deflate);
     free (request->body);
+    free (request->id);
 
     free (request);
 }
@@ -1627,6 +1623,7 @@ relay_http_print_log_request (struct t_relay_http_request *request)
     weechat_log_printf ("    content_length. . . . . : %d", request->content_length);
     weechat_log_printf ("    body_size . . . . . . . : %d", request->body_size);
     weechat_log_printf ("    body. . . . . . . . . . : '%s'", request->body);
+    weechat_log_printf ("    id. . . . . . . . . . . : '%s'", request->id);
 }
 
 /*
@@ -1641,7 +1638,7 @@ relay_http_print_log_response (struct t_relay_http_response *response)
     weechat_log_printf ("    http_version. . . . . . : '%s'", response->http_version);
     weechat_log_printf ("    return_code . . . . . . : %d", response->return_code);
     weechat_log_printf ("    message . . . . . . . . : '%s'", response->message);
-    weechat_log_printf ("    headers . . . . . . . . : 0x%lx (hashtable: '%s')",
+    weechat_log_printf ("    headers . . . . . . . . : %p (hashtable: '%s')",
                         response->headers,
                         weechat_hashtable_get_string (response->headers, "keys_values"));
     weechat_log_printf ("    content_length. . . . . : %d", response->content_length);
